﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
//using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using Buddy.BehaviorTree;
using Buddy.Common;
using Buddy.CommonBot;
using Buddy.Navigation;
using Buddy.Common.Math;
using Buddy.Swtor;
using Buddy.Swtor.Objects;

using Action = Buddy.BehaviorTree.Action;

namespace SlaveCollar
{
    public partial class Form1 : Form
    {
        public static string MasterName = "";
        public static float FollowDistance = 1.5f;

        private static TorCharacter Master;

        protected static Buddy.Swtor.Objects.TorPlayer Me { get { return BuddyTor.Me; } }

        public Form1()
        {
            InitializeComponent();
            numericUpDown1.Value = (int)(FollowDistance * 10);
        }

        private void button2_Click(object sender, EventArgs e)
        {
            MasterName = comboBox1.Items[comboBox1.SelectedIndex].ToString();
            Master = Me.PartyMembers().FirstOrDefault(p => p.Name == MasterName);

            if (Master != null)
            {
                Logging.Write("Master found: " + MasterName);

                try
                {
                    TreeHooks.Instance.Hooks.FirstOrDefault(h => h.Key == "RepopCorpse").Value.Insert(0, WaitToRepop());
                }
                catch
                {
                    Logging.Write("Unable to insert WaitToRepop()");
                }

                Logging.Write("Replacing Out of Combat Hook with Slave OOC Behavior");
                TreeHooks.Instance.ReplaceHook("Combat_OOC", SlaveOOCBehavior);
                Logging.Write("Replacing Combat Hook with Slave Combat Behavior");
                TreeHooks.Instance.ReplaceHook("Combat_Combat", SlaveCombatBehavior);
            }
            else
            {
                Logging.Write("No Master found :(");
            }

        }

        private void button1_Click(object sender, EventArgs e)
        {
            var party = Me.PartyMembers(true).ToList();
            comboBox1.Items.Clear();
            foreach (TorCharacter c in party)
            {
                comboBox1.Items.Add(c.Name);
            }
        }

        private void button3_Click(object sender, EventArgs e)
        {
            Logging.Write("Replacing Out of Combat Hook with Master OOC Behavior");
            TreeHooks.Instance.ReplaceHook("Combat_OOC", MasterOOCBehavior);
            Logging.Write("Replacing Combat Hook with Master Combat Behavior");
            TreeHooks.Instance.ReplaceHook("Combat_Combat", MasterCombatBehavior);
        }

        private void numericUpDown1_ValueChanged(object sender, EventArgs e)
        {
            FollowDistance = ((float)numericUpDown1.Value) / 10;
        }

        public static Composite CloseDistance(float range)
        {
            if (Me.CurrentTarget != null && Me.CurrentTarget.Distance <= range)
            {
                Navigator.MovementProvider.StopMovement();
                return new PrioritySelector();
            }

            return new Decorator(ret => Me.CurrentTarget != null,
                CommonBehaviors.MoveAndStop(location => Me.CurrentTarget.Position, range, true));
        }

        public static Composite WaitToRepop()
        {
            return new Action(delegate
            {
                if (Me.IsDead && !AllDead)
                    return RunStatus.Success;
                else
                    return RunStatus.Failure;
            });
        }

        private static bool AllDead
        {
            get
            {
                return !Me.PartyMembers().Any(p => p.IsDead);
            }
        }

        private static Composite CompanionHandler
        {
            get
            {
                return new PrioritySelector(
                   new Decorator(ret => Me.Companion != null && Me.Companion.IsDead && Me.CompanionUnlocked > 0,
                        new PrioritySelector(
                            Spell.WaitForCast(),
                            CommonBehaviors.MoveAndStop(location => Me.Companion.Position, 0.2f, true),
                            Spell.Cast("Revive Companion", on => Me.Companion, when => Me.Companion.Distance <= 0.2f)
                            ))
                   );
            }
        }

        public static TorCharacter ReviveTarget
        {
            get
            {
                return Me.PartyMembers().FirstOrDefault(p => p.IsDead);
            }
        }

        public static Composite RevivePartyMember
        {
            get
            {
                return new PrioritySelector(
                   new Decorator(ret => ReviveTarget != null,
                        new PrioritySelector(
                            Spell.WaitForCast(),
                            CommonBehaviors.MoveAndStop(location => Me.Companion.Position, 0.2f, true),
                            Spell.Cast("Revive", on => ReviveTarget, when => ReviveTarget.Distance <= 0.2f)
                            ))
                   );
            }
        }

        public static Composite GetMasterTarget
        {
            get
            {
                return new Action(delegate
                {
                    if (Master != null && Master.CurrentTarget != null && (Me.CurrentTarget == null || Me.CurrentTarget != Master.CurrentTarget))
                        Master.CurrentTarget.Target();

                    return RunStatus.Failure;
                });
            }
        }

        public static Composite BRez
        {
            get
            {
                return new PrioritySelector(new Decorator(ret => ReviveTarget != null,
                    new PrioritySelector(
                        Spell.Cast("Revival", ret => ReviveTarget, ret => AbilityManager.HasAbility("Revival")),
                        Spell.Cast("Reanimation", ret => ReviveTarget, ret => AbilityManager.HasAbility("Reanimation")),
                        Spell.Cast("Heartrigger Patch", ret => ReviveTarget, ret => AbilityManager.HasAbility("Heartrigger Patch")),
                        Spell.Cast("Resuscitation Probe", ret => ReviveTarget, ret => AbilityManager.HasAbility("Resuscitation Probe")),
                        Spell.Cast("Onboard AED", ret => ReviveTarget, ret => AbilityManager.HasAbility("Onboard AED")))));
            }
        }
        
        public static Composite SlaveOOCBehavior
        {
            get
            {
                return new Decorator(ret => !BuddyTor.Me.InCombat, 
                            new PrioritySelector(
                                RevivePartyMember,
                                GetMasterTarget,
                                CloseDistance(FollowDistance),
                                RoutineManager.Current.OutOfCombat));
            }
        }

        public static Composite SlaveCombatBehavior
        {
            get
            {
                return new Decorator(ret => BuddyTor.Me.InCombat,
                            new PrioritySelector(
                                BRez,
                                GetMasterTarget,
                                RoutineManager.Current.Combat));
            }
        }

        public static Composite MasterOOCBehavior
        {
            get
            {
                return new Decorator(ret => !BuddyTor.Me.InCombat,
                            new PrioritySelector(
                                RevivePartyMember,
                                RoutineManager.Current.OutOfCombat));
            }
        }

        public static Composite MasterCombatBehavior
        {
            get
            {
                return new Decorator(ret => BuddyTor.Me.InCombat,
                            new PrioritySelector(
                                BRez,
                                RoutineManager.Current.Combat));
            }
        }

    }
}
