﻿using System;
using System.Collections.Generic;
using System.Timers;
using System.Linq;
using System.Text;
using Buddy.Common;
using Buddy.BehaviorTree;
using Buddy.Common.Math;
using Buddy.CommonBot;
using Buddy.Swtor;
using Buddy.Swtor.Objects;

using Action = Buddy.BehaviorTree.Action;

namespace SlaveCollar
{
    public static class Spell
    {
        private static Buddy.Swtor.Objects.TorPlayer Me { get { return BuddyTor.Me; } }
        public delegate TorCharacter UnitSelectionDelegate(object context);
        public delegate T Selection<out T>(object context);

        public static Composite WaitForCast()
        {
            return new Decorator(ret => BuddyTor.Me.IsCasting,
                new Action(ret => RunStatus.Success));
        }
        #region Cast - by name

        public static Composite Cast(string spell, Selection<bool> reqs = null)
        {
            return Cast(spell, ret => Me.CurrentTarget, reqs);
        }

        public static Composite Cast(string spell, UnitSelectionDelegate onUnit, Selection<bool> reqs = null)
        {
            return
                new Decorator(
                    ret => (onUnit != null && onUnit(ret) != null && (reqs == null || reqs(ret)) && CastCheck(spell, onUnit(ret)) && onUnit(ret).InLineOfSight),
                    new PrioritySelector(
                        new Action(delegate
            {
                Logging.Write(">> Casting <<   " + spell);
                return RunStatus.Failure;
            }),
                        new Action(ret => AbilityManager.Cast(spell, onUnit(ret))))

                );
        }

        public static bool CastCheck(string spell, TorCharacter onUnit)
        {
            var abl = AbilityManager.KnownAbilities.FirstOrDefault(a => a.Name == spell);

            if (abl != null)
                Logging.Write("[" + onUnit.Name + "][" + abl.Name + "][" + Me.IsAbilityReady(abl, onUnit).ToString() + "]");

            return AbilityManager.CanCast(spell, onUnit);
        }

        public static Composite MultiDot(string dot, string debuff)
        {
            TorCharacter MultiDotTarget = ObjectManager.GetObjects<TorCharacter>().Where(t =>
                        t.InCombat
                        && !t.IsDead
                        && t.IsHostile
                        && !t.IsStunned
                        && !t.HasDebuff(debuff)).FirstOrDefault();

            if (MultiDotTarget == null)
                return new PrioritySelector();
            else
            {
                Logging.Write(" <-----> MultiDot Target Casting <-----> ");
                return Cast(dot, on => MultiDotTarget);
            }

        }

        public static Composite Debuff(string spell)
        {
            return Cast(spell, ret => Me.CurrentTarget != null && Me.CurrentTarget.IsHostile && !Me.CurrentTarget.HasDebuff(spell));
        }

        public static Composite CastOnGround(string spell, Selection<bool> reqs = null)
        {
            return
                new Decorator(
                    ret =>
                    (reqs == null || reqs(ret)) && Me.CurrentTarget != null,
                    CastOnGround(spell, ctx => Me.CurrentTarget.Position, ctx => true));
        }

        public static Composite CastOnGround(string spell, CommonBehaviors.Retrieval<Vector3> location, BooleanValueDelegate requirements)
        {
            return
                new Decorator(
                    ret =>
                    requirements != null && requirements(ret) && location != null && location(ret) != Vector3.Zero &&
                    AbilityManager.CanCast(spell, BuddyTor.Me.CurrentTarget),
                    new Action(ret => AbilityManager.Cast(spell, location(ret))));
        }
        #endregion

        #region Buff


        public static Composite Buff(string spell, Selection<bool> reqs = null)
        {
            return
                new Decorator(
                    ret => (reqs == null || reqs(ret)) && !Me.HasBuff(spell),
                    Spell.Cast(spell, ret => Me, ret => true));

        }
        #endregion

    }
}

